#ifndef E_UTIL_H
#include "eutil.h"
#endif



/*
--------------------------------------------------------------------------------
                                  CONSTRUCTOR
--------------------------------------------------------------------------------
*/
extern ETexture::ETexture( void )
{
    /*  Init the texture */
    eTexture = NULL;

    /*  Init texture dimensions */
    eWidth = 9;
    eHeight = 0;

    /*  Initial dimensions */
    ePosX = 0;
    ePosY = 0;
}


/*
--------------------------------------------------------------------------------
                                   DESTRUCTOR
--------------------------------------------------------------------------------
*/
extern ETexture::~ETexture( void )
{
    /*  Free the data... you know you want to */
    free_texture();
}


/*
--------------------------------------------------------------------------------
                             LOAD_TEXTURE_FROM_FILE

             Create a texture from a file loaded from a given path
--------------------------------------------------------------------------------
*/
extern bool ETexture::load_texture_from_file( const char *path )
{
    /*  Free the texture if it exists */
    free_texture();

    /*  Create an SDL surface from the given image */
    SDL_Surface *tempSurface = IMG_Load( path );
    if( tempSurface == NULL )
    {
        printf("Could not create surface from %s.  IMG Error:  %s\n",
                path, IMG_GetError() );
        return( false );
    }

    /*  Create a texture from the loaded surface */
    eTexture = SDL_CreateTextureFromSurface( gRenderer, tempSurface );
    if( eTexture == NULL )
    {
        printf("Unable to create texture from %s.  SDL Error:  %s\n",
                path, SDL_GetError() );

        /*  Free the temporary surface */
        SDL_FreeSurface( tempSurface );

        return( false );
    }

    /*  Get the texture dimensions */
    eWidth = tempSurface->w;
    eHeight = tempSurface->h;

    /*  Free the temporary surface */
    SDL_FreeSurface( tempSurface );

    /*  Return success */
    return( true );
}


/*
--------------------------------------------------------------------------------
                           CREATE_TEXTURE_FROM_STRING

                      Create a texture from a given string
--------------------------------------------------------------------------------
*/
extern bool ETexture::create_texture_from_string( const char *string,
        TTF_Font *font, SDL_Color textColor )
{
    /*  Get rid of pre-existing texture */
    free_texture();

    /*  Create text surface (blended) */
    SDL_Surface *tempSurface = TTF_RenderText_Blended( font, string,
            textColor );
    if( tempSurface == NULL )
    {
        printf("Unable to render text surface from '%s'.  TTF Error:  %s\n",
                string, TTF_GetError() );
        return( false );
    }

    /*  Create texture from surface pixels */
    eTexture = SDL_CreateTextureFromSurface( gRenderer, tempSurface );
    if( eTexture == NULL )
    {
        printf("Unable to create texture from the rendered text surface!  \
                SDL Error:  %s\n", SDL_GetError() );

        /*  Free the temporary surface */
        SDL_FreeSurface( tempSurface );

        return( false );
    }

    /*  Get texture dimensions from the temporary surface */
    eWidth = tempSurface->w;
    eHeight = tempSurface->h;

    /*  Get rid of the temporary surface */
    SDL_FreeSurface( tempSurface );

    /*  If we made it this far, we're golden */
    return( true );
}


/*
--------------------------------------------------------------------------------
                                  FREE_TEXTURE
                                  
                               Free texture data
--------------------------------------------------------------------------------
*/
/*  Free the texture data */
extern void ETexture::free_texture( void )
{
    /*  If the texture exists, destroy it, null the pointer, reset dimensions */
    if( eTexture != NULL )
    {
        SDL_DestroyTexture( eTexture );
        eTexture = NULL;
        eWidth = 0;
        eHeight = 0;
    }
}


/*
--------------------------------------------------------------------------------
                                     RENDER

                        Render the texture to the screen
--------------------------------------------------------------------------------
*/
/*  Render the texture to the screen */
extern void ETexture::render( int x, int y )
{
    if( x >= 0 )
        ePosX = x;
    if( y >= 0 )
        ePosY = y;

    /*  Destination rect for the renderer */
    SDL_Rect destR = { ePosX, ePosY, eWidth, eHeight };

    /*  Render to screen using global renderer */
    SDL_RenderCopy( gRenderer, eTexture, NULL, &destR );
}


/*
--------------------------------------------------------------------------------
                                   GET_WIDTH

                             Get the texture width
--------------------------------------------------------------------------------
*/
/*  Get the texture width */
extern int ETexture::get_width( void )
{
    return( eWidth );
}


/*
--------------------------------------------------------------------------------
                                   GET_HEIGHT

                             Get the texture height
--------------------------------------------------------------------------------
*/
/*  Get the texture height */
extern int ETexture::get_height( void )
{
    return( eHeight );
}


/*
--------------------------------------------------------------------------------
                                  SET_POSITION

                    Set the default position of the texture
--------------------------------------------------------------------------------
*/
/*  Set the default position of the texture */
extern void ETexture::set_position( int x, int y )
{
    ePosX = x;
    ePosY = y;
}


/*
--------------------------------------------------------------------------------
                                   GET_POS_X

                             Return the X position
--------------------------------------------------------------------------------
*/
/*  Get the X position */
extern int ETexture::get_pos_x( void )
{
    return( ePosX );
}


/*
--------------------------------------------------------------------------------
                                   GET_POS_Y

                             Return the Y position
--------------------------------------------------------------------------------
*/
/*  Get the Y position */
extern int ETexture::get_pos_y( void )
{
    return( ePosY );
}


/*
--------------------------------------------------------------------------------
                                   IS_INSIDE

                    Check if the mouse is inside the texture
--------------------------------------------------------------------------------
*/
extern bool ETexture::is_inside( void )
{
    /*  Find the mouse position */
    int x, y;
    SDL_GetMouseState( &x, &y );

    /*  Check if the mouse is within bounds of the current button */

    if( x < ePosX )            //  Left of button
        return( false );
    if( x > ePosX + eWidth )   //  Right of button
        return( false );
    if( y < ePosY )            //  Above button
        return( false );
    if( y > ePosY + eHeight )  //  Below button
        return( false );

    /*  We're good if we made it this far */
    return( true );

}
