#ifndef E_UTIL_H
#include "eutil.h"
#endif



/*
--------------------------------------------------------------------------------
                                  CONSTRUCTOR
--------------------------------------------------------------------------------
*/
extern ETime::ETime( int time )
{
    eTime = time;   //  Current time, in seconds since midnight
    eSaved = 0;     //  Currently 'saved' time (backup for future restoration)

    eHour = 0;      //  Hours, 0-23
    eMin = 0;       //  Minutes, 0-59
    eSec = 0;       //  Seconds, 0-59
}


/*
--------------------------------------------------------------------------------
                                  SET_TIME_TR

          Set the time to whatever was passed to the function (h/m/s)
--------------------------------------------------------------------------------
*/
extern void ETime::set_time_tr( int h, int m, int s )
{
    /*  If hour/min/sec were not less than zero, we use the args to set time */
    if( ! ( h < 0 && m < 0 && s < 0 ) )
    {
        /*  Set the hour / min / sec */
        eHour = h;
        eMin = m;
        eSec = s;

        /*  Correct it */
        correct_time();
    }

    /*  Set time */
    eTime = seconds_since_midnight( eHour, eMin, eSec );
}


/*
--------------------------------------------------------------------------------
                             SECONDS_SINCE_MIDNIGHT

            Determine the seconds since midnight according to h/m/s
--------------------------------------------------------------------------------
*/
extern int ETime::seconds_since_midnight( int h, int m, int s )
{
    int seconds = 0;            //  Init

    seconds += s;               //  Seconds
    seconds += m * 60;          //  Minutes
    seconds += h * 60 * 60;     //  Hours

    return( seconds );
}


/*
--------------------------------------------------------------------------------
                                 CALCULATE_TIME

          Calculate the time (seconds since midnight, hour, min, sec)
--------------------------------------------------------------------------------
*/
extern void ETime::calculate_time( time_t *rawTime )
{
    /*  Get the time struct */
    struct tm *cTime = localtime( rawTime );

    /*  Calculate and set seconds since midnight */
    eTime = seconds_since_midnight( cTime->tm_hour, cTime->tm_min,
            cTime->tm_sec );

    /*  Set the hour, min and sec variables */
    eHour = cTime->tm_hour;
    eMin = cTime->tm_min;
    eSec = cTime->tm_sec;
}


/*
--------------------------------------------------------------------------------
                                 TRANSLATE_TIME

                     Get hour/min/sec from current time int
--------------------------------------------------------------------------------
*/
extern void ETime::translate_time( void )
{
    /*  The current time, in seconds since midnight */
    int sec = eTime;

    /*  Boil it down to hours, minutes and seconds */

    int min = sec / 60;
    sec %= 60;

    int hour = min / 60;
    min %= 60;

    /*  Set our variables */
    eHour = hour;
    eMin = min;
    eSec = sec;
}


/*
--------------------------------------------------------------------------------
                                  CORRECT_TIME

                       Make sure the time stays in bounds
--------------------------------------------------------------------------------
*/
extern void ETime::correct_time( void )
{
    /*  Correct the hour */
    if( eHour > 23 )
        eHour = 0;
    else if( eHour < 0 )
        eHour = 23;

    /*  Correct the minute */
    if( eMin > 59 )
        eMin = 0;
    else if( eMin < 0 )
        eMin = 59;

    /*  Correct the second */
    if( eSec > 59 )
        eSec = 0;
    else if( eSec < 0 )
        eSec = 59;
}


/*
--------------------------------------------------------------------------------
                                   SAVE_TIME

                            Backup the current time
--------------------------------------------------------------------------------
*/
extern void ETime::save_time( void )
{
    eSaved = eTime;
}


/*
--------------------------------------------------------------------------------
                                  RESTORE_TIME

                            Restore time from backup
--------------------------------------------------------------------------------
*/
extern void ETime::restore_time( void )
{
    eTime = eSaved;
}


/*  Convert time to usable string.  Must be passed an array with enough chars */
/*
--------------------------------------------------------------------------------
                                  TIME_TO_STR

           Make a string from the current hours, minutes and seconds
--------------------------------------------------------------------------------
*/
extern char* ETime::time_to_str( char *string )
{
    /*  If we're using standard format */
    if( format12 == true )
    {
        /*  Convert time to STD format */
        int hour = eHour;
        if( hour > 12 )
            hour -= 12;

        if( hour == 0 )
            hour = 12;

        /*  Make the string */
        if( eHour > 11 )
            sprintf( string, "%02d:%02d:%02d PM", hour, eMin, eSec );
        else
            sprintf( string, "%02d:%02d:%02d AM", hour, eMin, eSec );
    }

    /*  If we're using 24-hour format (default) */
    else
        sprintf( string, "%02d:%02d:%02d", eHour, eMin, eSec );

    /*  Return the finished string */
    return( string );
}


/*
--------------------------------------------------------------------------------

                               ACCESSOR FUNCTIONS

--------------------------------------------------------------------------------
*/


/*  Return current time (seconds since midnight) */
extern int ETime::get_time( void )
{
    return( eTime );
}


/*  Set the time to whatever was passed */
extern void ETime::set_time( int time )
{
    if( time > (MAX_SECONDS) )
        eTime = (MAX_SECONDS);
    else
        eTime = time;
}


/*  Return the hour */
extern int ETime::get_hour( void )
{
    return( eHour );
}

/*  Return the minute */
extern int ETime::get_min( void )
{
    return( eMin );
}

/*  Return the second */
extern int ETime::get_sec( void )
{
    return( eSec );
}

/*  Set the hour */
extern void ETime::set_hour( int hour )
{
    eHour = hour;
    correct_time();
}

/*  Set the minute */
extern void ETime::set_min( int min )
{
    eMin = min;
    correct_time();
}


/*  Set the second */
extern void ETime::set_sec( int sec )
{
    eSec = sec;
    correct_time();
}



/*
--------------------------------------------------------------------------------

                                 TICK FUNCTIONS

--------------------------------------------------------------------------------
*/

/*  Tick the hour up or down */
extern void ETime::tick_hour( int tick )
{
    eHour += tick;
    correct_time();
}

/*  Tick the minute up or down */
extern void ETime::tick_min( int tick )
{
    eMin += tick;
    correct_time();
}

/*  Tick the second up or down */
extern void ETime::tick_sec( int tick )
{
    eSec += tick;
    correct_time();
}
