/*------------------------------------------------------------------------------
 *  This is where we declare our include files, enums, class headers and all the
 *  global stuff we need in the form of externs.
 *----------------------------------------------------------------------------*/
#ifndef E_UTIL_H
#define E_UTIL_H

#define xstr(s) str(s)
#define str(s) #s

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <string>
#include <sys/stat.h>
#include <sys/types.h>
#include <SDL2/SDL.h>
#include <SDL2/SDL_image.h>
#include <SDL2/SDL_mixer.h>
#include <SDL2/SDL_ttf.h>

#define SCREEN_WIDTH 800
#define SCREEN_HEIGHT 520

#define MAX_SECONDS 86400

#define VERSION "0.92"


/*  Include our custom classes */
#ifndef E_CLASSES_H
#include "eclasses.h"
#endif  //  E_CLASSES_H



/*
--------------------------------------------------------------------------------
                                    EXTERNS
--------------------------------------------------------------------------------
*/

/*  Our wonderful global renderer */
extern SDL_Renderer *gRenderer;

/*  Our global window */
extern SDL_Window *gWindow;

/*  SDL Colors */
extern SDL_Color textColor;         //  White
extern SDL_Color alarmTextColor;    //  Orange-ish
extern SDL_Color activateColor;     //  Yellow

/*  SDL Rects */
extern SDL_Rect sysTimeRect;        //  Rect drawn around system time
extern SDL_Rect alarmTimeRect;      //  Rect drawn around alarm time
extern SDL_Rect timeSlotRect;       //  Rect drawn around time slot (time scrn)
extern SDL_Rect presetRect;         //  Rect drawn around preset buttons

/*  TTF Fonts */
extern TTF_Font *gFonts[];          //  Regular fonts
extern TTF_Font *gFontsMono[];      //  Monospace fonts

/*  Integers */
extern int currentScreen;       //  Which screen the user is currently on
extern int changingTime;        //  Which time is being edited
extern int currentToolTip;      //  Tooltip to be displayed
extern int changingTimeFocus;   //  Which 'time slot' is highlighted
extern Uint32 heldTicks;        //  How long a key has been held

/*  Used to get system time */
extern time_t sysTimeRaw;
extern struct tm *sysTime;

/*  Character arrays */
extern char sysTimeStr[];       //  The string for the system time
extern char indexFile[];        //  The file which lists textures to load
extern char dataDir[];          //  The directory in which the textures reside
extern char soundFile[];        //  The sound file to use

/*  Global booleans */
extern bool format12;       //  Whether or not to use 12-Hour format; default no
extern bool ring;                   //  Whether or not the alarm is 'ringing'

/*  Textures */
extern ETexture *gButtonTextures[];         //  Graphical button textures
extern ETexture *gTextTextures[];           //  Text textures
extern ETexture *gToolTips[];               //  Tooltip textures

/*  Buttons */
extern EButton *gMainButtons[];             //  Main screen buttons
extern EPresetButton *gPresetButtons[];     //  Preset buttons
extern EButton *gTimeButtons[];             //  Time screen buttons
extern EPresetButton *gAlarmButton;         //  The alarm time button

/*  Our global times array */
extern ETime *gTimes[];



/*
--------------------------------------------------------------------------------
                                   FUNCTIONS
--------------------------------------------------------------------------------
*/

/*  Initialization functions */
void init_time_screen( int id );
void init_preset_times( void );
void init_alarm_time( void );
void init_time_buttons( void );
void init_alarm_screen( void );


/*  Layout functions */
void layout_preset_buttons( void );
void layout_graphical_buttons( void );
void layout_time_screen( void );
void layout_main_screen( void );
void layout_help_screen( void );
void layout_about_screen( void );
void layout_alarm_screen( void );


/*  Misc time stuff */
void change_time_string( int id );
void update_system_time( void );
char* update_system_time( char *sysTimeStr );


/*  Config stuff */
bool find_system_dir( void );
bool find_user_dir( void );
bool create_user_dir( void );
bool find_config_file( void );
bool read_config_file( void );
bool set_default_sound_file( void );
bool init_config( void );
bool write_config_file( void );



/*
--------------------------------------------------------------------------------
                                     ENUMS
--------------------------------------------------------------------------------
*/

/*  Which time slot is selected */
enum changingTimeFocusEnum
{
    FOCUS_HOUR,
    FOCUS_MINUTE,
    FOCUS_SECOND
};


/*  Font sizes */
enum globalFonts
{
    SIZE_TINY,
    SIZE_SMALL,
    SIZE_MEDIUM,
    SIZE_BIG,
    SIZE_HUGE,
    SIZE_BIGGEST,
    TOTAL_FONTS
};


/*  Which screen we're on */
enum screens
{
    SCREEN_MAIN,
    SCREEN_HELP,
    SCREEN_ABOUT,
    SCREEN_FORMAT,
    SCREEN_TIME,
    SCREEN_ALARM,
    TOTAL_SCREENS
};


/*  Times */
enum times
{
    TIME_1,
    TIME_2,
    TIME_3,
    TIME_4,
    TIME_5,
    TIME_6,
    TIME_7,
    TIME_8,
    TIME_9,
    TIME_10,
    TIME_ALARM,
    TIME_SYSTEM,
    TOTAL_TIMES
};


/*  Buttons on the time screen */
enum timeButtons
{
    T_BUTTON_UP1,
    T_BUTTON_UP2,
    T_BUTTON_UP3,
    T_BUTTON_DOWN1,
    T_BUTTON_DOWN2,
    T_BUTTON_DOWN3,
    T_BUTTON_YES1,
    T_BUTTON_NO1,
    TOTAL_TIME_BUTTONS
};


/*  Preset buttons on the main screen */
enum presetButtons
{
    PRESET1,
    PRESET2,
    PRESET3,
    PRESET4,
    PRESET5,
    PRESET6,
    PRESET7,
    PRESET8,
    PRESET9,
    PRESET10,
    TOTAL_PRESET_BUTTONS
};


/*  Graphical buttons, for to do the clicking on, yes? */
enum graphicalButtons
{
    T_BUTTON_MAIN,
    T_BUTTON_HELP,
    T_BUTTON_ABOUT,
    T_BUTTON_FORMAT,
    T_BUTTON_GO,
    T_BUTTON_UP,
    T_BUTTON_DOWN,
    T_BUTTON_YES,
    T_BUTTON_NO,
    TOTAL_GRAPHICAL_BUTTONS
};


/*  All of the graphical textures */
enum graphicalTextures
{
    T_GRAPHICAL_MAIN_ACTIVATED,
    T_GRAPHICAL_MAIN_DEACTIVATED,
    T_GRAPHICAL_HELP_ACTIVATED,
    T_GRAPHICAL_HELP_DEACTIVATED,
    T_GRAPHICAL_ABOUT_ACTIVATED,
    T_GRAPHICAL_ABOUT_DEACTIVATED,
    T_GRAPHICAL_FORMAT_ACTIVATED,
    T_GRAPHICAL_FORMAT_DEACTIVATED,
    T_GRAPHICAL_GO_ACTIVATED,
    T_GRAPHICAL_GO_DEACTIVATED,
    T_GRAPHICAL_UP_ACTIVATED,
    T_GRAPHICAL_UP_DEACTIVATED,
    T_GRAPHICAL_DOWN_ACTIVATED,
    T_GRAPHICAL_DOWN_DEACTIVATED,
    T_GRAPHICAL_YES_ACTIVATED,
    T_GRAPHICAL_YES_DEACTIVATED,
    T_GRAPHICAL_NO_ACTIVATED,
    T_GRAPHICAL_NO_DEACTIVATED,
    TOTAL_GRAPHICAL_TEXTURES
};


/*  All of the text textures */
enum textTextures
{
    T_TEXT_PRESET_1,
    T_TEXT_PRESET_2,
    T_TEXT_PRESET_3,
    T_TEXT_PRESET_4,
    T_TEXT_PRESET_5,
    T_TEXT_PRESET_6,
    T_TEXT_PRESET_7,
    T_TEXT_PRESET_8,
    T_TEXT_PRESET_9,
    T_TEXT_PRESET_10,
    T_TEXT_MAIN_SCREEN_TIME_ALARM,
    T_TEXT_MAIN_SCREEN_TIME_SYSTEM,
    T_TEXT_MAIN_SCREEN_TITLE,
    T_TEXT_MAIN_SCREEN_HINT1,
    T_TEXT_MAIN_SCREEN_HINT2,
    T_TEXT_MAIN_SCREEN_HINT3,
    T_TEXT_MAIN_SCREEN_HINT4,
    T_TEXT_MAIN_SCREEN_HINT5,
    T_TEXT_MAIN_SCREEN_HINT_ALARM,
    T_TEXT_MAIN_SCREEN_HINT_SYSTEM,
    T_TEXT_HELP_HINT1,
    T_TEXT_HELP_HINT2,
    T_TEXT_HELP_HINT3,
    T_TEXT_HELP_HINT4,
    T_TEXT_HELP_HINT5,
    T_TEXT_HELP_HINT6,
    T_TEXT_HELP_HINT7,
    T_TEXT_HELP_HINT8,
    T_TEXT_HELP_HINT9,
    T_TEXT_ABOUT_HINT1,
    T_TEXT_ABOUT_HINT2,
    T_TEXT_ABOUT_HINT3,
    T_TEXT_ABOUT_HINT4,
    T_TEXT_ABOUT_HINT5,
    T_TEXT_ABOUT_HINT6,
    T_TEXT_ALARM_SCREEN_SYSTEM,
    T_TEXT_ALARM_SCREEN_CURRENT,
    T_TEXT_ALARM_SCREEN_HINT1,
    T_TEXT_ALARM_SCREEN_HINT2,
    T_TEXT_ALARM_SCREEN_HINT3,
    T_TEXT_TIME_TITLE,
    T_TEXT_TIME_TIME,
    T_TEXT_TIME_HINT1,
    T_TEXT_TIME_HINT2,
    T_TEXT_TIME_HINT3,
    T_TEXT_TOOLTIP,
    TOTAL_TEXT_TEXTURES
};


/*  Button types */
enum buttonTypes
{
    BT_PRESET,
    BT_MAIN,
    BT_HELP,
    BT_ABOUT,
    BT_FORMAT,
    BT_GO,
    BT_PREVIEW,
    BT_UP_HOUR,
    BT_UP_MINUTE,
    BT_UP_SECOND,
    BT_DOWN_HOUR,
    BT_DOWN_MINUTE,
    BT_DOWN_SECOND,
    BT_YES,
    BT_NO,
    TOTAL_BUTTON_TYPES
};


/*  Tooltip currently displayed */
enum tooltips
{
    TOOLTIP_MAIN,
    TOOLTIP_HELP,
    TOOLTIP_ABOUT,
    TOOLTIP_FORMAT,
    TOOLTIP_GO,
    TOOLTIP_PRESET,
    TOOLTIP_SYSTEM,
    TOOLTIP_ALARM,
    TOTAL_TOOLTIPS
};

#endif
